#include "X3D_GLBuilderCoreVisitor.h"
#include "X3D_Scene.h"
#include "X3D_X3DNode.h"
#include "GL_Scene.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

GLBuilderCoreVisitor::GLBuilderCoreVisitor()
: CoreVisitor()
{
  // Enter functions.
  define(Recorder<Scene>::getEnterFunction(&GLBuilderCoreVisitor::enterScene));
  define(Recorder<X3DNode>::getEnterFunction(&GLBuilderCoreVisitor::enterX3DNode));

  // WalkOn functions.
  define(Recorder<X3DNode>::getWalkOnFunction(&GLBuilderCoreVisitor::walkOnX3DNode));
  
  // Leave functions.
  define(Recorder<X3DNode>::getLeaveFunction(&GLBuilderCoreVisitor::leaveX3DNode));
}

void GLBuilderCoreVisitor::enterScene(Scene *S) 
{
  GLBuilderStateVariables *stateVariables = Singleton<GLBuilderStateVariables>::getInstance();
  GL::X3DNode * GS = stateVariables->getNode(S);
  if (GS == 0)
  {
    GS = new GL::Scene();
    GS->setX3DReference(S);
    stateVariables->addCoupleNode(S, GS);
  }
  stateVariables->pushNode(GS); 
  
  if (chdir(S->getFileName().getPath()) != 0)
    cx3d << "GLBuilderCoreVisitor::enterScene: unable to set the current working directory to " << S->getFileName().getPath() << "!" << endl; 
}

void GLBuilderCoreVisitor::enterX3DNode(X3DNode *N) 
{
  GLBuilderStateVariables *stateVariables = Singleton<GLBuilderStateVariables>::getInstance();
  cx3d << "warning: no GL implementation for " << N->getTypeName() << " !" << endl;
  stateVariables->pushNode(0);
}

bool GLBuilderCoreVisitor::walkOnX3DNode(X3DNode *N, SFNode) 
{
  GLBuilderStateVariables *stateVariables = Singleton<GLBuilderStateVariables>::getInstance();
  return (stateVariables->getTop() != 0);
}

void GLBuilderCoreVisitor::leaveX3DNode(X3DNode *) 
{
  GLBuilderStateVariables *stateVariables = Singleton<GLBuilderStateVariables>::getInstance();
  GL::X3DNode * N = stateVariables->getTop();
  if (N != 0)
    N->update();
    
  stateVariables->popNode();
}

}
}
