#include "X3D_SceneSaverWriteCoreVisitor.h"
#include "X3D_Scene.h"
#include "X3D_X3DNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

SceneSaverWriteCoreVisitor::SceneSaverWriteCoreVisitor()
: CoreVisitor()
{
  // Enter functions.
  define(Recorder<X3DNode>::getEnterFunction(&SceneSaverWriteCoreVisitor::enterX3DNode));

  // Walk on functions.
  define(Recorder<X3DNode>::getWalkOnFunction(&SceneSaverWriteCoreVisitor::walkOnX3DNode));

  // Leave functions.
  define(Recorder<X3DNode>::getLeaveFunction(&SceneSaverWriteCoreVisitor::leaveX3DNode));
  define(Recorder<Scene>::getLeaveFunction(&SceneSaverWriteCoreVisitor::leaveScene));
}

void SceneSaverWriteCoreVisitor::enterX3DNode(X3DNode *N) 
{
  SceneSaverStateVariables *stateVariables = Singleton<SceneSaverStateVariables>::getInstance();

  stateVariables->newLine();
  stateVariables->pushCurrentLine();
  SFString &currentLine = stateVariables->getCurrentLine();
  
  if (stateVariables->findNode(N))
  {
    stateVariables->pushHasUSE(true);
    currentLine += "<" + N->getTypeName() + " USE=\"" + N->getName() + "\"";
  }
  else
  {
    stateVariables->pushHasUSE(false);    
    if (N->getName() != "")
      currentLine += "<" + N->getTypeName() + " DEF=\"" + N->getName() + "\"";
    else
      currentLine += "<" + N->getTypeName();  
    
    //N->write(currentLine);  
    N->writeAttributes(currentLine);  
    
    stateVariables->addNode(N);
  }
      
  stateVariables->addTab();
}

bool SceneSaverWriteCoreVisitor::walkOnX3DNode(X3DNode *, SFNode) 
{
  SceneSaverStateVariables *stateVariables = Singleton<SceneSaverStateVariables>::getInstance();

  //return true;
  return !(stateVariables->getHasUSE());  
}

void SceneSaverWriteCoreVisitor::leaveX3DNode(X3DNode *N) 
{
  SceneSaverStateVariables *stateVariables = Singleton<SceneSaverStateVariables>::getInstance();

  stateVariables->removeTab();
  
  if (stateVariables->hasChild())
    stateVariables->getOpeningLine() += "/>";  
  else
  {
    stateVariables->getOpeningLine() += ">";
    stateVariables->newLine();
    stateVariables->getCurrentLine() += "</" + N->getTypeName() + ">";
  }  
  
  stateVariables->popLine();
  stateVariables->popHasUSE();  
}

void SceneSaverWriteCoreVisitor::leaveScene(Scene *S) 
{
  SceneSaverStateVariables *stateVariables = Singleton<SceneSaverStateVariables>::getInstance();

  stateVariables->removeTab();
  
  if (stateVariables->hasChild())
    stateVariables->getOpeningLine() += "/>";  
  else
  {
    stateVariables->getOpeningLine() += ">";
    
    MFRoute routes = S->getRoutes();
    for (MFRoute::const_iterator it = routes.begin(); it != routes.end(); ++it)
    {
      stateVariables->newLine();
      stateVariables->getCurrentLine() += "<ROUTE fromField=\"" + (*it)->getFromField() 
                                             + "\" fromNode=\"" + (*it)->getFromNode()->getName() 
                                              + "\" toField=\"" + (*it)->getToField()
                                               + "\" toNode=\"" + (*it)->getToNode()->getName() + "\"/>";
    }

    stateVariables->newLine();
    stateVariables->getCurrentLine() += "</" + S->getTypeName() + ">";
  }  
  
  stateVariables->popLine();
  stateVariables->popHasUSE();  
}

}
}
