#include "X3D_WorldCoordTranslatorMulRenderingVisitor.h"
#include "X3D_Coordinate.h"
#include "X3D_Normal.h"
#include "X3D_X3DGeometryNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {


WorldCoordTranslatorMulRenderingVisitor::WorldCoordTranslatorMulRenderingVisitor()
: RenderingVisitor()
{
  // Enter functions.
  define(Recorder<Coordinate>::getEnterFunction(&WorldCoordTranslatorMulRenderingVisitor::enterCoordinate));
  define(Recorder<Normal>::getEnterFunction(&WorldCoordTranslatorMulRenderingVisitor::enterNormal));
}

void WorldCoordTranslatorMulRenderingVisitor::enterCoordinate(Coordinate *C) 
{
  WorldCoordTranslatorStateVariables *stateVariables = Singleton<WorldCoordTranslatorStateVariables>::getInstance();

  if (!stateVariables->findProcessedNode(C))
  {
    stateVariables->pushProcessedNode(C);
  
    SFMatrix34f transform = stateVariables->getMatrix();
  
    MFVec3f vertexArray = C->getPoint();
    for (MFVec3f::iterator itVertex = vertexArray.begin(); itVertex != vertexArray.end(); ++itVertex)
      *itVertex = SFVec3f(transform*SFPoint3f(*itVertex));

    C->setPoint(vertexArray);  
  }  
}

void WorldCoordTranslatorMulRenderingVisitor::enterNormal(Normal *N) 
{
  WorldCoordTranslatorStateVariables *stateVariables = Singleton<WorldCoordTranslatorStateVariables>::getInstance();
  
  if (!stateVariables->findProcessedNode(N))
  {
    stateVariables->pushProcessedNode(N);
    
    SFMatrix34f transform = stateVariables->getMatrix();
  
    MFVec3f normalArray = N->getVector();
    for (MFVec3f::iterator itNormal = normalArray.begin(); itNormal != normalArray.end(); ++itNormal)
    {
      *itNormal = (transform*(*itNormal));
      (*itNormal).normalize();
    }  

    N->setVector(normalArray);  
  }  
}

}
}
