#include "X3D_WorldCoordTranslatorShareCoreVisitor.h"
#include "X3D_X3DNode.h"
#include "X3D_Transform.h"
#include "X3D_Scene.h"
#include "Cloner.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {


WorldCoordTranslatorShareCoreVisitor::WorldCoordTranslatorShareCoreVisitor()
: CoreVisitor()
{
  define(Recorder<X3DNode>::getEnterFunction(&WorldCoordTranslatorShareCoreVisitor::enterX3DNode));
}

void WorldCoordTranslatorShareCoreVisitor::enterX3DNode(X3DNode *N) 
{
  WorldCoordTranslatorStateVariables *stateVariables = Singleton<WorldCoordTranslatorStateVariables>::getInstance();
  Cloner *cloner = Singleton<Cloner>::getInstance();

  if (!stateVariables->findProcessedNode(N))
  {
    stateVariables->pushProcessedNode(N);
  
    // Getting the partition of the parents for the transformation hierarchy.
    map<Transform *, MFNode> parentMap = getParentMap(N);
    map<Transform *, MFNode>::iterator it = parentMap.begin();
    
    while ((it != parentMap.end()) && ((*it).second.empty()))
      ++it;

    if (it != parentMap.end()) 
      ++it;
      
    while (it != parentMap.end())
    {
      if (!(*it).second.empty())
      {
        // cloning the sub-graph.
        X3DAbstractNode *clone = cloner->clone(N);
        // replacing Coordinate.
        MFNode parentList = (*it).second;
        for (MFNode::const_iterator itParent = parentList.begin(); itParent != parentList.end(); ++itParent)
        {
          (*itParent)->removeChild(N);
          (*itParent)->setChild(clone);
        }  
      }  
      ++it;
    }
  }  
}

map<Transform *, MFNode> WorldCoordTranslatorShareCoreVisitor::getParentMap(X3DNode *N)
{
  map<Transform *, MFNode> parentMap;
  
  MFNode parentList = N->getParentList();
  
  for (MFNode::const_iterator it = parentList.begin(); it != parentList.end(); ++it)
  {
    list<Transform *> result = getTransformParent(*it);
    for (list<Transform *>::const_iterator itTransform = result.begin(); itTransform != result.end(); ++itTransform)
      parentMap[*itTransform].push_back(*it);
  }
  
  return parentMap;
}

list<Transform *> WorldCoordTranslatorShareCoreVisitor::getTransformParent(X3DAbstractNode *N)
{
  list<Transform *> result;

  if (dynamic_cast<Scene *>(N) != 0)
  {
    result.push_back(0);
    return result;
  }
  
  if (dynamic_cast<Transform *>(N) != 0)
  {
    result.push_back(static_cast<Transform *>(N));
    return result;
  }
  MFNode parentList = N->getParentList();
  
  for (MFNode::const_iterator it = parentList.begin(); it != parentList.end(); ++it)
  {
    list<Transform *> parentResult = getTransformParent(*it);
    for (list<Transform *>::const_iterator itTransform = parentResult.begin(); itTransform != parentResult.end(); ++itTransform)
      result.push_back(*itTransform);
  }
  
  return result;
}

}
}
