#include "JPGImageType.h"
#include "Image.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef JPEG

extern "C" {
#include <setjmp.h>
#include <jpeglib.h>
}

#endif

#include <iostream>

using namespace std;

namespace X3DTK {

JPGImageType::JPGImageType()
{
}

bool JPGImageType::load(const char *fileName, Image *image)
{
  bool retCode = false;

#ifdef JPEG

  struct local_error_mgr
  {
    struct jpeg_error_mgr pub;
    jmp_buf setjmp_buffer;
  };

  unsigned char *destData;
  typedef struct local_error_mgr *local_error_ptr;
  struct local_error_mgr jerr;
  struct jpeg_decompress_struct cinfo;
  FILE *infile;
  JSAMPARRAY buffer;

  int row_stride;

  if((infile = fopen(fileName, "rb")))
  {
    cinfo.err = jpeg_std_error(&jerr.pub);
    if(setjmp(jerr.setjmp_buffer))
    {
      jpeg_destroy_decompress(&cinfo);
      fclose(infile);
      return 0;
    }

    jpeg_create_decompress(&cinfo);
    jpeg_stdio_src(&cinfo, infile);
    jpeg_read_header(&cinfo, TRUE);
    jpeg_start_decompress(&cinfo);

    GLenum GLFormat = 0;
    
    switch (cinfo.output_components)
    {
      case 0:
        GLFormat = 0;
        break;
      case 1:
        GLFormat = GL_LUMINANCE;
        break;  
      case 2:
        GLFormat = GL_LUMINANCE_ALPHA;
        break;  
      case 3:
        GLFormat = GL_RGB;
        break;  
      case 4:
        GLFormat = GL_RGBA;
        break;  
      default:
        GLFormat = 0;  
    }
    
    if (image->allocate(cinfo.output_width, cinfo.output_height, GLFormat, GL_UNSIGNED_BYTE))
    {
      unsigned char *data = image->getData();
      destData = data + cinfo.output_width*cinfo.output_height*cinfo.output_components;
      row_stride = cinfo.output_width*cinfo.output_components;
      buffer = (*cinfo.mem->alloc_sarray) ((j_common_ptr) & cinfo, JPOOL_IMAGE, row_stride, 1);
      
      while(cinfo.output_scanline < cinfo.output_height)
      {
        destData -= row_stride;
        jpeg_read_scanlines(&cinfo, buffer, 1);
        memcpy(destData, *buffer, row_stride);
      }
 
      retCode = true;
    }
    else
      retCode = false;

    jpeg_finish_decompress(&cinfo);
    jpeg_destroy_decompress(&cinfo);
    fclose(infile);
  }

#else
  cx3d << "sorry, jpeg not supported by your platform!" << endl;
#endif

 
  return retCode;
}

bool JPGImageType::save(const char *fileName, Image *image, float quality)
{
  bool    retCode = false;
  
 #ifdef JPEG
    struct local_error_mgr
    {
        struct jpeg_error_mgr   pub;
        jmp_buf                 setjmp_buffer;
    };

    typedef struct local_error_mgr  *local_error_ptr;

    struct local_error_mgr          jerr;
    struct jpeg_compress_struct     cinfo;
    FILE                            *outfile;
    JSAMPARRAY                      buffer;
    unsigned char                   *data;

    if((outfile = fopen(fileName, "wb")) == NULL)
    {
        fprintf(stderr, "can't open %s\n", fileName);
        return retCode;
    }

    cinfo.err = jpeg_std_error(&jerr.pub);
    if(setjmp(jerr.setjmp_buffer))
    {
        jpeg_destroy_compress(&cinfo);
        fclose(outfile);
        return false;
    }

    jpeg_create_compress(&cinfo);
    jpeg_stdio_dest(&cinfo, outfile);

    cinfo.image_width = image->getWidth();
    cinfo.image_height = image->getHeight();
    
    cinfo.input_components = image->computePixelSize(image->getGLFormat(), GL_UNSIGNED_BYTE);
    cinfo.in_color_space = (cinfo.input_components == 1) ? JCS_GRAYSCALE : JCS_RGB;

    jpeg_set_defaults(&cinfo);
    jpeg_set_quality(&cinfo, static_cast<int>(quality*100.0f), TRUE);
    jpeg_start_compress(&cinfo, TRUE);

    buffer = &data;
    
    unsigned char *newData = image->resizeData(image->getWidth(), image->getHeight(), GL_UNSIGNED_BYTE);
    while(cinfo.next_scanline < cinfo.image_height)
    {
        data = newData +
            (image->getHeight() - 1 - cinfo.next_scanline) *
            image->getWidth() *
            cinfo.input_components;
        jpeg_write_scanlines(&cinfo, buffer, 1);
    }

    delete newData;
    
    jpeg_finish_compress(&cinfo);
    jpeg_destroy_compress(&cinfo);
    fclose(outfile);

     retCode = true;
#else
  cx3d << "sorry, jpeg not supported by your platform!" << endl;
#endif

  return retCode;
}

}
